@@ -6,7 +6,8 @@ from .wind_rose_test_case import WindRoseTestCase
 from .kriger_test_case import KrigerTestCase
 from .tow_calculator_test_case import TOWCalculatorTestCase
 from .vector_layer_test_case import VectorLayerTestCase
+from .hourly_grid_test_case import HourlyGridTestCase
 
 # algorithm tests
 from .grid_to_point_test_case import GridToPointTestCase
-from .raster_distance_to_vector_test_case import RasterDistanceToVectorTestCase
+#from .raster_distance_to_vector_test_case import RasterDistanceToVectorTestCase
