@@ -24,7 +24,7 @@ class RasterBand:
     self._pixel_h = None
     self._storage_method = None
     self._extent = None
-    self._metadata = None
+    self._metadata = {}
 
     # projection stuff 
     self._datum = None
@@ -338,7 +338,7 @@ class RasterBand:
     # get epsg and proj string 
     epsg_code = crs.to_epsg()
     proj_string = crs.to_proj4()
-    logger.info(f"Loaded raster with Datum: {datum} EPSG: {epsg_code}")
+    logger.debug(f"Loaded raster with Datum: {datum} EPSG: {epsg_code}")
     
     self._datum = datum
     self._epsg_code = epsg_code
@@ -353,6 +353,14 @@ class RasterBand:
   def ul(self):
     return (self._x_origin, self._y_origin)
 
+  # return two lists of arrays
+  # x coordinates and y coordinates
+  # note: these should return the center points
+  def get_coordinate_arrays(self):
+    x = np.array([self._x_origin + (j * self._pixel_w) for j in range(self._cols)])
+    y = np.array([self._y_origin - (i * self._pixel_h) for i in range(self._rows)])
+    return x, y
+
   def lr(self):
     return (self._x_origin + self._cols * self._pixel_w,
       self._y_origin - self._rows * self._pixel_h)
@@ -391,6 +399,9 @@ class RasterBand:
   def get_metadata(self):
     return self._metadata
 
+  def add_metadata(self, field, name):
+    return self.set_metadata(field, name)
+
   def set_metadata(self, field, name):
     assert field is not None, "Field must not be None"
     self._metadata[field] = name
@@ -425,22 +436,25 @@ class RasterBand:
     center_y = y - (self._pixel_h/2) # we go from the top down
     return (center_x, center_y)
 
-  def write_to_file(self, output_filename, compress=False, overwrite=True):
-    return self.save_to_file(output_filename, compress, overwrite)
+  def write_to_file(self, output_filename, compress=False, overwrite=True, dtype=gdal.GDT_Float32):
+    return self.save_to_file(output_filename, compress, overwrite, dtype)
 
-  def save_to_file(self, output_filename, compress=False, overwrite=True):
+  def save_to_file(self, output_filename, compress=False, overwrite=True, dtype=gdal.GDT_Float32):
     if os.path.exists(output_filename) and not overwrite:
       raise ValueError(f"File exists: {output_filename}")
     if self._projection_wkt is None:
       raise ValueError("No projection set on raster")
-  
+ 
+    if dtype == 'uint8':
+      dtype = gdal.GDT_Byte
+
     options = []
     if compress:
       options = ['COMPRESS=LZW', 'BIGTIFF=YES']
 
     driver = gdal.GetDriverByName('GTiff')
     out_raster = driver.Create(output_filename, 
-      self._cols, self._rows, 1, gdal.GDT_Float32,
+      self._cols, self._rows, 1, dtype,
       options=options)
     out_raster.SetGeoTransform((self._x_origin, self._pixel_w, 0, 
       self._y_origin, 0, -self._pixel_h))
@@ -478,6 +492,27 @@ class RasterBand:
     # then replace the existing data with this new data
     self._band.WriteArray(row_data, 0, row_number)
 
+  # return two lists of arrays
+  # x coordinates and y coordinates
+  # note: these should return the center points
+  def get_coordinate_arrays(self, return_centroids=False):
+    x = np.array([self._x_origin + (j * self._pixel_w) for j in range(self._cols)])
+    y = np.array([self._y_origin - (i * self._pixel_h) for i in range(self._rows)])
+
+    # if they want the centroids adjust both values
+    # by width/2 so we get the point centerpoint
+    if return_centroids:
+      x = x + (self._pixel_w / 2)
+      y = y - (self._pixel_h / 2)
+
+    return x, y
+    
+  def flatten(self):
+    return self.values().ravel()
+
+  # return a list of rows, where each row is a list of tuples
+  # the first element of the tuple is the x,y coordinate
+  # the second element is the value
   def rows(self, return_centroids=False):
     for i in range(self._rows):
       y = self._y_origin - (i * self._pixel_h)
@@ -535,9 +570,102 @@ class RasterBand:
     data = self.values()
     # count rows that are NOT nan
     nodata_count = np.count_nonzero(np.isnan(data))
-
     return nodata_count / data.size
 
+  # split this raster into chunks of size rows x cols
+  # and create a new raster object for each one that we make
+  # note that the size is a function of the size of the grid
+  # so like 2,2 will return 2 grid boxes by 2 grid boxes, etc
+  # this is regardless of the projection
+  def chunk_fixed_size(self, rows=1000, cols=1000, overlap_rows=0, overlap_cols=0, overlap_method='nan'):
+    self._throw_except_if_band_not_loaded()
+
+    # make sure the rows and cols are valid
+    if rows > self._rows or cols > self._cols:
+      raise ValueError("Invalid chunk size")
+
+    # make sure boht are ints
+    rows = int(rows)
+    cols = int(cols)
+    overlap_rows = int(overlap_rows)
+    overlap_cols = int(overlap_cols)
+
+    # now we need to iterate over the raster and create
+    # new rasters for each chunk, but let's make sure we
+    # use the native gdal library for that
+    for i in range(0, self._rows, rows):
+      start_row = max(i - overlap_rows, 0)
+      end_row = min(i + rows + overlap_rows, self._rows)
+      rows_to_read = end_row - start_row
+      expected_rows = min(i + rows, self._rows) - max(i, 0) + (2 * overlap_rows)
+
+      for j in range(0, self._cols, cols):
+        start_col = max(j - overlap_cols, 0)
+        end_col = min(j + cols + overlap_cols, self._cols)
+        cols_to_read = end_col - start_col
+        expected_cols = min(j + cols, self._cols) - max(j, 0) + (2 * overlap_cols)
+
+        data = self._band.ReadAsArray(start_col, start_row, 
+          cols_to_read, rows_to_read)
+        data = np.where(data == self._nodata, np.nan, data)
+
+        logger.debug(f"data shape = {data.shape}")
+
+        # now we have to determine how to pad in the case
+        # where haven't queried the rows from the data
+        # which is the case when we're at the edge
+        if rows_to_read < expected_rows:
+          if start_row == 0:
+            #print(f"padding top to {expected_rows}")
+            if overlap_method == 'nan':
+              data = np.pad(data, ((expected_rows - rows_to_read, 0), (0, 0)), 
+                'constant', constant_values=np.nan)
+            elif overlap_method == 'reflect':
+              data = np.pad(data, ((expected_rows - rows_to_read, 0), (0, 0)), 
+                'reflect')
+          elif end_row == self._rows:
+            #print(f"padding bottom to {expected_rows}")
+            if overlap_method == 'nan':
+              data = np.pad(data, ((0, expected_rows - rows_to_read), (0, 0)), 
+                'constant', constant_values=np.nan)
+            elif overlap_method == 'reflect':
+              data = np.pad(data, ((0, expected_rows - rows_to_read), (0, 0)), 
+                'reflect')
+        if cols_to_read < expected_cols:
+          if start_col == 0:
+            #print(f"padding left to {expected_cols}")
+            if overlap_method == 'nan':
+              data = np.pad(data, ((0, 0), (expected_cols - cols_to_read, 0)), 
+                'constant', constant_values=np.nan)
+            elif overlap_method == 'reflect':
+              data = np.pad(data, ((0, 0), (expected_cols - cols_to_read, 0)), 
+                'reflect')
+          elif end_col == self._cols:
+            #print(f"padding right to {expected_cols}")
+            if overlap_method == 'nan':
+              data = np.pad(data, ((0, 0), (0, expected_cols - cols_to_read)), 
+                'constant', constant_values=np.nan)
+            elif overlap_method == 'reflect':
+              data = np.pad(data, ((0, 0), (0, expected_cols - cols_to_read)), 
+                'reflect')
+
+        #print((rows_to_read, expected_rows, cols_to_read, expected_cols))
+        #print(data.shape)
+        #print((expected_rows, expected_cols))
+        new_raster = RasterBand()
+        new_raster.blank_raster(expected_rows, expected_cols,
+          (self._pixel_w, self._pixel_h),
+          (self._x_origin + (j * self._pixel_w), 
+           self._y_origin - (i * self._pixel_h)))
+        new_raster.load_array(data, self._nodata)
+        assert new_raster.width() == expected_cols, "Cols do not match"
+        assert new_raster.height() == expected_rows, "Rows do not match"
+        if self._projection_wkt is not None:
+          new_raster.set_projection(self._projection_wkt)
+        new_raster.add_metadata('OVERLAP_ROWS', overlap_rows)
+        new_raster.add_metadata('OVERLAP_COLS', overlap_cols)
+        yield new_raster
+
   # return the values in the raster as a numpy array
   # note the values shape is rows + cols
   def values(self):
