@@ -13,6 +13,7 @@ import logging
 import datetime
 import pytz
 from .wind_rose import WindRose
+from .hourly_grid import HourlyGrid
 from .tow_calculator import TOWCalculator
 from .helpers import should_value_be_none, simple_confirm_value_in_range, \
     validate_dt_or_convert_to_datetime_obj
@@ -43,6 +44,8 @@ class WeatherStation:
     self.air_humidity_values = {}
     self.air_dewpoint_c_values = {}
 
+    self.precip_values = {}
+
     self.location = {'latitude': None,
       'longitude': None, 'elevation': None}
 
@@ -51,6 +54,9 @@ class WeatherStation:
     # functions there
     self.wind_rose = WindRose(8, precision)
 
+    # support precipitation with the hourly grid
+    self.precip_grid = HourlyGrid()
+
     # pm25 and pm10 are ug/m3
     self.pm_25_values = {}
     self.pm_10_values = {}
@@ -176,6 +182,31 @@ class WeatherStation:
     if self.tow is not None and value is not None:
       self.tow.add_temperature(value, dt)
 
+  # precipitation is more complicated, because it involves a range
+  # of times, not just a single one. but gets reported as a single one
+  # value = mm of precipitation
+  # number_mins = number of minutes the precipitation occurred
+  # dt = datetime object
+  # example: 10mm over last 60 minutes
+  def add_precip_mm(self, value, number_mins, dt):
+    dt = validate_dt_or_convert_to_datetime_obj(dt)
+    value = should_value_be_none(value)
+    payload = {
+      'value': value,
+      'number_mins': number_mins
+    }
+    self.precip_values[dt] = payload
+
+    # if we have a ONE HOUR object, then add the precipitation
+    if number_mins == 60:
+      self.precip_grid.add(dt, value)
+
+  # just for precip typos
+  def add_precipitation_mm(self, value, number_mins, dt):
+    self.add_precip_mm(value, number_mins, dt)
+
+  # value = 0 to 100
+  # dt = datetime object
   def add_humidity(self, value, dt):
     value = should_value_be_none(value)
     dt = validate_dt_or_convert_to_datetime_obj(dt)
@@ -300,6 +331,21 @@ class WeatherStation:
   def _max(self, values):
     return round(max([v[1] for v in values if v[1] is not None]), self._precision)
 
+  def _sum(self, values):
+    return round(sum([v[1] for v in values if v[1] is not None]), self._precision)
+
+  # precip has to use a special data structure because of how reporting
+  # that comes in is hourly and we want to be able to report on it hourly
+  def get_precipitation_mm(self, measure='SUM'):
+    if measure == 'SUM':
+      return self.precip_grid.get_total()
+    elif measure == 'MEAN':
+      return self.precip_grid.get_mean()
+    elif measure == 'MAX':
+      return self.precip_grid.get_max()
+    elif measure == 'MIN':
+      return self.precip_grid.get_min()
+
   def get_temp_c(self, measure='MEAN'):
     measure = measure.upper()
     return self._get_value_metric('air_temp_c_values', measure)
@@ -329,6 +375,8 @@ class WeatherStation:
       return self.ozone_ppb_values
     elif field_name == 'so2':
       return self.so2_values
+    elif field_name in ('precipitation', 'precip'):
+      return self.precip_values
     else:
       raise ValueError(f"Invalid field name: {field_name}")
 
@@ -413,6 +461,8 @@ class WeatherStation:
       return self._max(field_values)
     elif measure == 'MIN':
       return self._min(field_values)
+    elif measure == 'SUM':
+      return self._sum(field_values)
     else:
       raise ValueError(f"Invalid measure: {measure}")
 
@@ -487,6 +537,15 @@ class WeatherStation:
           'date_range': self.get_date_range('air_pressure_hpa'),
           'full_year': self.is_full_year_of_data('air_pressure_hpa')
         },
+        'precipitation': {
+          'count': len(self.precip_values),
+          'date_range': self.get_date_range('precipitation'),
+          'full_year': self.is_full_year_of_data('precipitation'),
+          'sum': self.get_precipitation_mm('SUM'),
+          'min': self.get_precipitation_mm('MIN'),
+          'max': self.get_precipitation_mm('MAX'),
+          'yearly': self.precip_grid.get_total_by_year_detailed()
+        },
         'wind': {
           'speed': {
             'vector_mean': speed,
@@ -538,6 +597,12 @@ class WeatherStation:
       payload['air']['time_of_wetness'] = self.tow.get_averages()
       payload['air']['time_of_wetness']['by_year'] = self.tow.get_years()
 
+    # confirm we can dump to JSON
+    try:
+      json.dumps(payload)
+    except Exception as e:
+      raise ValueError(f"Cannot serialize to JSON: {e}. Payload: {payload}")
+
     return json.dumps(payload)
 
   def get_elevation(self):
